﻿/******************************************************************************
 * File Name			: DynamicBuilder.cs
 * Author				: Naga Jamini Mungara
 * Date of Creation		: 23/03/2014
 *  
 * ____________________________________________________________________________
 *****************************************************************************/
using System;
using System.Reflection;
using System.Reflection.Emit;
using System.Data;

namespace DataContextDemo.Common
{
    /// <summary>
    /// Class to create the mapped object based on the datatable reader
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class DynamicBuilder<T>
    {
        private delegate T Load(IDataReader dataReader);
        private Load handler;

        private DynamicBuilder() { }

        public T Build(IDataReader reader)
        {
            return handler(reader);
        }

        public static DynamicBuilder<T> CreateBuilder(IDataReader reader)
        {
            DynamicBuilder<T> result = new DynamicBuilder<T>();
            DynamicMethod method = new DynamicMethod("DynamicCreate", typeof(T), new[] { typeof(IDataReader) }, typeof(T), true);

            ILGenerator generator = method.GetILGenerator();
            generator.DeclareLocal(typeof(T));


            generator.Emit(OpCodes.Newobj, typeof(T).GetConstructor(Type.EmptyTypes));
            generator.Emit(OpCodes.Stloc_0);

            MethodInfo getValue = reader.GetType().GetMethod("get_Item", new[] { typeof(int) });

            for (int i = 0; i < reader.FieldCount; i++)
            {

                PropertyInfo propertyInfo = typeof(T).GetProperty(reader.GetName(i),BindingFlags.IgnoreCase | BindingFlags.Public | BindingFlags.Instance);

                if (propertyInfo == null || propertyInfo.GetSetMethod() == null) continue;

                bool isNullable = false;

                if (propertyInfo.PropertyType.Name.ToLower().Contains("nullable")) isNullable = true;

                Label label = generator.DefineLabel();
                generator.Emit(OpCodes.Ldarg_0);
                generator.Emit(OpCodes.Ldc_I4, i);
                generator.Emit(OpCodes.Callvirt, typeof(IDataRecord).GetMethod("IsDBNull"));
                generator.Emit(OpCodes.Brtrue, label);
                generator.Emit(OpCodes.Ldloc_0);
                generator.Emit(OpCodes.Ldarg_0);
                generator.Emit(OpCodes.Ldc_I4, i);
                generator.Emit(OpCodes.Callvirt, getValue);

                Type _type = reader.GetFieldType(i);

                if (isNullable) generator.Emit(OpCodes.Unbox_Any, GetNullableType(_type));
                else generator.Emit(OpCodes.Unbox_Any, _type);

                generator.Emit(OpCodes.Callvirt, propertyInfo.GetSetMethod());
                generator.MarkLabel(label);

            }

            generator.Emit(OpCodes.Ldloc_0);
            generator.Emit(OpCodes.Ret);
            result.handler = (Load)method.CreateDelegate(typeof(Load));

            return result;
        }

        private static Type GetNullableType(Type type)
        {
            Type result = null;

            if (type == typeof(bool)) result = typeof(Nullable<bool>);
            if (type == typeof(byte)) result = typeof(Nullable<byte>);
            if (type == typeof(DateTime)) result = typeof(Nullable<DateTime>);
            if (type == typeof(decimal)) result = typeof(Nullable<decimal>);
            if (type == typeof(double)) result = typeof(Nullable<double>);
            if (type == typeof(float)) result = typeof(Nullable<float>);
            if (type == typeof(Guid)) result = typeof(Nullable<Guid>);
            if (type == typeof(Int16)) result = typeof(Nullable<Int16>);
            if (type == typeof(Int32)) result = typeof(Nullable<Int32>);
            if (type == typeof(Int64)) result = typeof(Nullable<Int64>);
            return result;
        }
    }
}